#ifndef __ISDS_ISDS_PRIV_H__
#define __ISDS_ISDS_PRIV_H__

/* Structures not to export outside library */
#include "../config.h"
#include "isds.h"
#include <curl/curl.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xpath.h>
#include <libxml/xpathInternals.h>
#include <libxml/xmlsave.h>
#include "gettext.h"

#define _(x) ((const char *) dgettext(PACKAGE, (x)))

#define SOAP_NS "http://schemas.xmlsoap.org/soap/envelope/"
#define SOAP2_NS "http://www.w3.org/2003/05/soap-envelope"
#define ISDS_NS "http://isds.czechpoint.cz/v20"
#define SISDS_INCOMING_NS "http://isds.czechpoint.cz/v20/message"
#define SISDS_OUTGOING_NS "http://isds.czechpoint.cz/v20/SentMessage"
#define SISDS_DELIVERY_NS "http://isds.czechpoint.cz/v20/delivery"
#define SCHEMA_NS "http://www.w3.org/2001/XMLSchema"

/* Used to choose proper name space for message elements.
 * See register_namespaces(). */
typedef enum {
    MESSAGE_NS_UNSIGNED,
    MESSAGE_NS_SIGNED_INCOMING,
    MESSAGE_NS_SIGNED_OUTGOING,
    MESSAGE_NS_SIGNED_DELIVERY
} message_ns_type;

/* Global variables.
 * Allocated in isds_init() and deallocated in isds_cleanup(). */
unsigned int log_facilities;
isds_log_level log_level;

/* End of global variables */

/* Context */
struct isds_ctx {
    unsigned int timeout;   /* milliseconds */
    char *url;              /* URL of the ISDS web service */
    char *username;
    char *password;
    char *client_certificate;
    char *private_key;
    CURL *curl;             /* CURL session handle */
    _Bool *tls_verify_server;   /* Verify the server? */
    isds_progress_callback progress_callback;  /* Call it during
                                                   communication with server.
                                                   NULL for nothing */
    void *progress_callback_data;       /* Application provided argument
                                           for progress_callback */
    char *tls_ca_file;      /* File name with CA certificates */
    char *tls_ca_dir;       /* Directory name with CA certificates */
    char *long_message;     /* message buffer */
};

/* Stores message into context' long_message buffer.
 * Application can pick the message up using isds_long_message().
 * NULL @message truncates the buffer but does not deallocate it.
 * @message is coded in locale encoding */
isds_error isds_log_message(struct isds_ctx *context, const char *message);

/* Appends message into context' long_message buffer.
 * Application can pick the message up using isds_long_message().
 * NULL message has void effect. */
isds_error isds_append_message(struct isds_ctx *context, const char *message);

/* Stores formated message into context' long_message buffer.
 * Application can pick the message up using isds_long_message(). */
isds_error isds_printf_message(struct isds_ctx *context,
        const char *format, ...);

/* Log @message in class @facility with log @level into global log. @message
 * is printf(3) formating string, variadic arguments may be neccessary.
 * For debugging purposes. */
isds_error isds_log(const isds_log_facility facility,
        const isds_log_level level, const char *message, ...);

/* Makes known all relevant namespaces to given XPath context
 * @xpat_ctx is XPath context
 * @message_ns selects propper message name space. Unsisnged and signed
 * messages differs.
 * prefix and to URI ISDS_NS */
isds_error register_namespaces(xmlXPathContextPtr xpath_ctx,
        const message_ns_type message_ns);

#endif
